include particle.e
include font.e
include sound.ew
include globals.e
include get.e

-----------------------------------------------------------------------------------------------

constant X_RADIUS = 1.15,
	 Y_RADIUS = 0.4,
	 CHERRY_BOMB_SIZE = 200

-----------------------------------------------------------------------------------------------

integer
	hiscore_col,
    	hiscore_dcol,
	
	-- Bounds for the substring of the message to be printed.
	scroller_first_visible,
    	scroller_last_visible,
	scroller_col,
	
	latest_entry,
	hiscore_file,
	
	hiscore_music_playing,
	hiscore_reshape_id
	

atom
	scroller_xpos,
     	scroller_angle,
	hiscore_colchange

sequence
	-- Some fireworks
	bomb1,bomb2,bomb3,

	entries,
	hiscore_name,

	-- Trig-tables used when printing the characters in an ellipse on the
	-- name-entry screen.
	cos_table,					 
      	sine_table,
	sine_table2


-----------------------------------------------------------------------------------------------

-- Scrolling message
constant scroller = 
"ThAnk you for plAying Tetrix^.  OriginAl C++ code by Mic And Windom.  "&
"EuphoriA code by Mic.  GrAphics by Mic.  The music wAs downloAded from"&
" VArious mod-Archives on The neT.  Sound effects were tAken from the G"&
"AmeBoy Version of Tetris, And from two SNES gAmes by KonAmi; Chibi MAr"&
"uko-chAn And Jikkyo oshAberi PArodius......"

constant hiscore_name_chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-. `@"

constant new_cherry_bomb = {0,{},0,0,{0,0,0},{0,0,0},{0,0,0}}
constant
	TTB = 1,
	PARTICLES = 2


integer name_char, name_chars_length,name_index

latest_entry = -1
name_char = 0
hiscore_music_playing = 0
entries = repeat({0,0,0},5)
	
-----------------------------------------------------------------------------------------------

-- Create and initialize a new cherry bomb (well, ok it's not such a good name but
-- who gives a turkey..?)
global function cherry_bomb_init()
	sequence bomb
	sequence p
	atom x,y
	
	bomb = new_cherry_bomb
	p = repeat(new_particle,CHERRY_BOMB_SIZE)

	x = rand(1000)/50.0 - 10.0
	y = rand(500)/50.0 - 5.0
	bomb[COL] = rand(12)

	for i=1 to CHERRY_BOMB_SIZE do
		p[i][ALIVE] = 1
		p[i][LIFE] = 1.0
		p[i][FADE] = (rand(100)/900.0) + 0.006
		p[i][POS] = {x,y,rand(100)/200.0 - 0.25}
		p[i][DIR][1] = (rand(60)-32.0) * 2.1
		p[i][DIR][2] = (rand(60)-30.0) * 2.5
		p[i][DIR][3] = (rand(60)-30.0) * 1.25
		p[i][GRAVITY] = {0.0,-0.8,-0.04}
		p[i][COL] = part_pal1[bomb[COL]]
	end for
	bomb[PARTICLES] = p
	
	bomb[TTB] = rand(60) + 30		-- Cycles until burst

	-- Randomize position 
	bomb[POS] = {rand(1000) / 50.0 - 10.0,
		     -3.0,
		     rand(6) / 2.0 - 40.0}

	-- Randomize speed 
	bomb[DIR] = {(rand(60)-31.0) * (x / -4.0) / 200.0,
		     rand(30)/150.0+0.5,
		     (rand(60)-30.0)/75.0}
	if (bomb[DIR][3]>=-0.05) then
		bomb[DIR][1] *= (1.0 - bomb[DIR][3] / 2.0)
		bomb[DIR][2] *= (1.0 - bomb[DIR][3] / 2.0)
	end if
	
	-- Set gravity
	if bomb[DIR][3] = 0.0 then
		-- Avoid division by zero
		bomb[GRAVITY] = {-0.0002,
				 -0.009,
				 0}
	else
		bomb[GRAVITY] = {-0.0002,
				 -0.009,	
				 (bomb[DIR][3] / fabs(bomb[DIR][3])) / 300.0}
	end if		 

	return bomb
end function



-- Set the position of all particles to the current position of the bomb
global function cherry_bomb_start_explosion(sequence bomb)
	for i=1 to CHERRY_BOMB_SIZE do
		bomb[PARTICLES][i][POS] = bomb[POS]
	end for
	return bomb
end function



global function cherry_bomb_draw(sequence bomb, atom zoom, atom res)
	atom x,y,z,x_speed,y_speed
	sequence pp
	
	-- Is the bomb about to burst/explode ?
	if (bomb[TTB] <=0 and bomb[TTB]>-1000) then
		bomb = cherry_bomb_start_explosion(bomb)
		bomb[TTB] = -1000
		return bomb
	end if

	if (bomb[TTB] = -1000) then
		c_proc(glDisable,{GL_DEPTH_TEST})
		pp = draw_particles(bomb[PARTICLES], CHERRY_BOMB_SIZE, prttex, zoom, res, 0.5, 0.0, bury_id) 
		if not pp[1] then
			-- Reinitialize when there are no live particles left
			bomb = cherry_bomb_init()
		else
			bomb[PARTICLES] = pp[2]
		end if
	else
		-- Draw a burning firework soaring through the air
		c_proc(glEnable,{GL_DEPTH_TEST})
		c_proc(glDepthFunc,{GL_GEQUAL})
		c_proc(glEnable,{GL_TEXTURE_2D})
		c_proc(glBindTexture,{GL_TEXTURE_2D,prttex})
		c_proc(glEnable,{GL_BLEND})
		c_proc(glBlendFunc,{GL_SRC_ALPHA,GL_ONE})

		x = bomb[POS][1]
		y = bomb[POS][2]
		z = bomb[POS][3]
		x_speed = bomb[DIR][1]
		y_speed = bomb[DIR][2]
		
		-- Draw a white blob at the current position
		c_proc(glColor4f,{1,1,1,0.9})
		c_proc(glBegin,{GL_TRIANGLE_STRIP})
			c_proc(glTexCoord2f,{1.0, 0.0})
			c_proc(glVertex3f,bomb[POS]+{0.5,0.5,zoom})
			c_proc(glTexCoord2f,{0.0, 0.0})
			c_proc(glVertex3f,bomb[POS]+{-0.5,0.5,zoom})
			c_proc(glTexCoord2f,{1.0, 1.0})
			c_proc(glVertex3f,bomb[POS]+{0.5,-0.5,zoom})
			c_proc(glTexCoord2f,{0.0, 1.0})
			c_proc(glVertex3f,bomb[POS]+{-0.5,-0.5,zoom})
		c_proc(glEnd,{})

		-- ..then a yellow blob at the previous position
		c_proc(glColor4f,{1,1,0.4,0.9})
		c_proc(glBegin,{GL_TRIANGLE_STRIP})
			c_proc(glTexCoord2f,{1.0, 0.0})
			c_proc(glVertex3f,{x+0.5-x_speed, y+0.5-y_speed, z+zoom})
			c_proc(glTexCoord2f,{0.0, 0.0})
			c_proc(glVertex3f,{x-0.5-x_speed, y+0.5-y_speed, z+zoom})
			c_proc(glTexCoord2f,{1.0, 1.0})
			c_proc(glVertex3f,{x+0.5-x_speed, y-0.5-y_speed, z+zoom})
			c_proc(glTexCoord2f,{0.0, 1.0})
			c_proc(glVertex3f,{x-0.5-x_speed, y-0.5-y_speed, z+zoom})
		c_proc(glEnd,{})

		-- ..and finally a red/orange blob at the second previous position (approximately)
		c_proc(glColor4f,{1,0.5,0.1,0.9})
		c_proc(glBegin,{GL_TRIANGLE_STRIP})
			c_proc(glTexCoord2f,{1.0, 0.0})
			c_proc(glVertex3f,{x+0.5-(x_speed*2.0), y+0.5-(y_speed*2.0), z+zoom})
			c_proc(glTexCoord2f,{0.0, 0.0})
			c_proc(glVertex3f,{x-0.5-(x_speed*2.0), y+0.5-(y_speed*2.0), z+zoom})
			c_proc(glTexCoord2f,{1.0, 1.0})
			c_proc(glVertex3f,{x+0.5-(x_speed*2.0), y-0.5-(y_speed*2.0), z+zoom})
			c_proc(glTexCoord2f,{0.0, 1.0})
			c_proc(glVertex3f,{x-0.5-(x_speed*2.0), y-0.5-(y_speed*2.0), z+zoom})
		c_proc(glEnd,{})

		bomb[POS] += bomb[DIR]*speed_factor
		bomb[DIR] += bomb[GRAVITY]*speed_factor
		bomb[TTB] -= speed_factor
		c_proc(glDepthFunc,{GL_LESS})
		c_proc(glDisable,{GL_BLEND})
		c_proc(glDisable,{GL_TEXTURE_2D})
	end if
	
	return bomb
	
end function


-----------------------------------------------------------------------------------------------


function hiscore_display()
	atom s,t,x,y,an,y_pos
	integer c,col

	t = time()
	speed_factor = (t-last_frame_time)/0.01785  -- aim at ~56 fps
	last_frame_time = t
	
	c_proc(glClear,{GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT})

	-- Draw the evening sky
	c_proc(glBegin,{GL_QUADS})
		c_proc(glColor3f,{0.2,0.2,0.9})		-- Blue
		c_proc(glVertex3f,{-60,2,-60})
		c_proc(glVertex3f,{60,2,-60})
		c_proc(glColor3f,{0.3,0.0,0.3})		-- Dark purple
		c_proc(glVertex3f,{60,12,-60})
		c_proc(glVertex3f,{-60,12,-60})

		c_proc(glColor3f,{0.3,0.0,0.3})
		c_proc(glVertex3f,{-60,12,-60})
		c_proc(glVertex3f,{60,12,-60})
		c_proc(glColor3f,{0,0,0})
		c_proc(glVertex3f,{60,20,-60})
		c_proc(glVertex3f,{-60,20,-60})
	c_proc(glEnd,{})

	-- Draw some stars in the sky
	c_proc(glBegin,{GL_POINTS})
		c_proc(glColor3f,{1,1,1})		-- Bright stars
		c_proc(glVertex3f,{-20,20,-55})
		c_proc(glVertex3f,{5,16,-55})
		c_proc(glVertex3f,{-10,17.5,-55})
		c_proc(glVertex3f,{13,19,-55})
		c_proc(glVertex3f,{-1,21,-55})
		c_proc(glVertex3f,{21,15,-55})
		c_proc(glColor3f,{0.8,0.8,0.8})		-- Semi-bright stars
		c_proc(glVertex3f,{10,20,-55})
		c_proc(glVertex3f,{-12.5,15,-55})
		c_proc(glVertex3f,{17,15.5,-55})
		c_proc(glVertex3f,{3,19,-55})
		c_proc(glColor3f,{0.5,0.5,0.5})		-- Dark stars
		c_proc(glVertex3f,{-4,14,-55})
		c_proc(glVertex3f,{0.5,18,-55})
		c_proc(glVertex3f,{18,14,-55})
		c_proc(glEnd,{})

	c_proc(glEnable,{GL_TEXTURE_2D})

	-- Draw the forest silhouette
	c_proc(glBindTexture,{GL_TEXTURE_2D,forest})
	c_proc(glEnable,{GL_BLEND})
	c_proc(glBlendFunc,{GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA})
	c_proc(glColor3f,{0,0,0})		-- Make the silhouette completely black
	c_proc(glBegin,{GL_QUADS})
		c_proc(glTexCoord2f,{0,0})
		c_proc(glVertex3f,{-2.15,1.75,-5})
		c_proc(glTexCoord2f,{0,1})
		c_proc(glVertex3f,{-2.15,-2.25,-5})
		c_proc(glTexCoord2f,{1,1})
		c_proc(glVertex3f,{2.15,-2.25,-5})
		c_proc(glTexCoord2f,{1,0})
		c_proc(glVertex3f,{2.15,1.75,-5})
	c_proc(glEnd,{})

	-- Draw the fireworks
	bomb1 = cherry_bomb_draw(bomb1, -20, 500)
	bomb2 = cherry_bomb_draw(bomb2, -16, 430)
	bomb3 = cherry_bomb_draw(bomb3, -18, 450)

	-- Draw the hiscore list
	c_proc(glDisable,{GL_DEPTH_TEST})
	y_pos = 0.12
	for i=1 to 5 do
		if (i=latest_entry) then
			c_proc(glColor3f,hiscore_pal[hiscore_col+1])
		else
			c_proc(glColor4f,{0.4,0.5,1,1})
		end if
		putsxyz(entries[i][1], font4, {-0.375,y_pos,-1.5}, 32, {0.09,0.07}, 0.07)
		putsxyz(entries[i][3], font4, {-0.07,y_pos,-1.5}, 32, {0.09,0.07}, 0.07)
		y_pos -= 0.09
	end for

	-- Draw the sine scroller
	x = scroller_xpos
	an = scroller_angle
	-- Determine which slice of the string to display
	if (scroller_xpos > 0.8) then
			scroller_first_visible = -1
	elsif (scroller_xpos >= -0.8) then
		scroller_first_visible = 1
		scroller_last_visible = scroller_first_visible + floor((0.8-scroller_xpos)/0.042)
	else
		-- This is the case where x_pos < -0.8 (i.e. the string has started going off the left
		-- side of the window.
		scroller_first_visible = floor((scroller_xpos+0.8)/-0.048)
		an += scroller_first_visible * 0.15
		x += scroller_first_visible * 0.042
		scroller_first_visible += 1
		scroller_last_visible = scroller_first_visible + floor((0.8-x)/0.042)
	end if

	if (scroller_first_visible >= 1) then
		col = scroller_col

		for j=scroller_first_visible to scroller_last_visible do
			if (j>length(scroller)) then exit end if
			c = scroller[j]
			c_proc(glColor3f,part_pal1[col+1])
			if (c != ' ') then
				c -= ' '
				-- Calculate the texture coordinates
				s = and_bits(c,15)*16
				t = and_bits(c,#F0)
				s /= 256.0
				t /= 256.0
				-- Calculate the y coordinate
				y = sin(an)*0.04-0.48

				c_proc(glBegin,{GL_QUADS})
					c_proc(glTexCoord2f,{s, t})
					c_proc(glVertex3f,{x,y, -1.5})
					c_proc(glTexCoord2f,{s, t+0.05859375})	-- 0.05859375 = 15/256
					c_proc(glVertex3f,{x, y - 0.045, -1.5})
					c_proc(glTexCoord2f,{s + 0.05859375, t + 0.05859375})
					c_proc(glVertex3f,{x + 0.045,y - 0.045, -1.5})
					c_proc(glTexCoord2f,{s + 0.05859375, t})
					c_proc(glVertex3f,{x + 0.045, y, -1.5})
				c_proc(glEnd,{})
			end if
			x += 0.042		-- Horizontal character spacing
			an += 0.15		-- Increase the angle (about 8.6 degrees)
			col += 1
			if (col=12) then col=0 end if
		end for
	end if
	-- Update the scroller parameters
	scroller_xpos -= 0.0039*speed_factor
	if scroller_xpos < -(0.042*(length(scroller)+17)) then
		scroller_xpos = 0.75
	end if
	scroller_angle += 0.035*speed_factor		-- 0.035 = ~2 degrees
--	if and_bits(hiscore_colchange,1) then
--		scroller_col = remainder(scroller_col+1,12)
--	end if
	
	c_proc(glEnable,{GL_DEPTH_TEST})
	c_proc(glDisable,{GL_BLEND})
	c_proc(glDisable,{GL_TEXTURE_2D})

	c_proc(glutSwapBuffers,{})

	-- Update the hiscore list parameters
	hiscore_colchange -= speed_factor
	if (hiscore_colchange<=0) then
		scroller_col = remainder(scroller_col+1,12)
		hiscore_col += hiscore_dcol
		if ((hiscore_col=0) or (hiscore_col=11)) then
			hiscore_dcol = -hiscore_dcol
		end if
		hiscore_colchange = 2.0
	end if
	
	return 0
end function
hiscore_display_cb = call_back(routine_id("hiscore_display"))



global procedure hiscore_setup()
	-- Load some textures
	prttex = texture({"data\\particle.bmp"})
	forest = texture({"data\\forest.bmp",255,0,0})
	font4 = font("data\\font4.bmp")
	pane = texture({"data\\pane.bmp"})

	-- Open the highscore list
	hiscore_file = open("hiscore","rb")
	if hiscore_file=-1 then
		-- If no highscore list exists, try to create one
		hiscore_file = open("hiscore","wb")
		if hiscore_file=-1 then
			puts(1,"Failed to create highscore file!")
			abort(0)
		end if
		-- Fill the list with some bogus values
		puts(hiscore_file,"MIC")
		puts(hiscore_file,int_to_bytes(10000))
		puts(hiscore_file,"MIC")
		puts(hiscore_file,int_to_bytes(5000))
		puts(hiscore_file,"ABC")
		puts(hiscore_file,int_to_bytes(1500))
		puts(hiscore_file,"TET")
		puts(hiscore_file,int_to_bytes(375))
		puts(hiscore_file,"RIX")
		puts(hiscore_file,int_to_bytes(200))
		
		close(hiscore_file)
		hiscore_file = open("hiscore","rb")
		if hiscore_file=-1 then
			-- Give up and abort
			puts(1,"Failed to open highscore file!")
			abort(0)
		end if
	end if	

	for i=1 to 5 do
		entries[i][1] = get_bytes(hiscore_file,3)
		entries[i][3] = get_bytes(hiscore_file,4)
		entries[i][2] = bytes_to_int(entries[i][3])
		entries[i][3] = sprintf("%06d",entries[i][2])
	end for
	close(hiscore_file)
end procedure


procedure hjoy_hiscore()
	if joybtn[3] or joybtn[4] then
		StopMod(songs[5])
		call_proc(start_new_game_id,{})

	elsif joybtn[5] then
		options_restore_displayfunc = display_cb
		options_restore_reshapefunc = reshape_cb
		options_restore_keyboardfunc = keyboard_cb
		options_restore_specfunc = 0 --spec_func_cb
		options_restore_joyhandler = joyhandler_id
		options()
	end if
end procedure
constant hjoy_hiscore_id = routine_id("hjoy_hiscore")


function hiscore_input(integer key_code,atom x,atom y)
	key_code = and_bits(key_code,#FF)	-- BUGFIX 2003-06-20
	
	if (key_code=13) then
		StopMod(songs[5])
		hiscore_music_playing = 0
		call_proc(start_new_game_id,{})

	elsif (key_code=27) then
		options_restore_displayfunc = hiscore_display_cb
		options_restore_reshapefunc = hiscore_reshape_cb
		options_restore_keyboardfunc = hiscore_input_cb
		options_restore_specfunc = 0 --spec_func_cb
		options_restore_joyhandler = joyhandler_id
		options_restore_reshape_id = hiscore_reshape_id
		options()		
	end if
	
	return 0
end function
hiscore_input_cb = call_back(routine_id("hiscore_input"))



function hiscore_reshape(integer w,integer h)
	c_proc(glViewport,{0,0,w,h})
	game_window = {w,h}

	c_proc(glMatrixMode,{GL_PROJECTION})
	c_proc(glLoadIdentity,{})

	c_proc(gluPerspective,{45.0,1, 0.1, 200.0})

	c_proc(glMatrixMode,{GL_MODELVIEW})
	c_proc(glLoadIdentity,{})

	return 0
end function
hiscore_reshape_id = routine_id("hiscore_reshape")
hiscore_reshape_cb = call_back(hiscore_reshape_id)



global procedure hiscore_prepare()
	integer i

	c_proc(glShadeModel,{GL_SMOOTH})
	c_proc(glClearColor,{0.0,0.0,0.0,0.0})
	c_proc(glClearDepth,{1.0})
	c_proc(glHint,{GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST})
	c_proc(glHint,{GL_POINT_SMOOTH_HINT,GL_NICEST})

	bomb1 = cherry_bomb_init()
	bomb2 = cherry_bomb_init()
	bomb3 = cherry_bomb_init()

	hiscore_col = 0
	hiscore_dcol = 1
	hiscore_colchange = 3
	scroller_xpos = 0.8
	scroller_angle = 0.0
	scroller_col = 0
	
	if not hiscore_music_playing then
		if not PlayModEx(songs[5],#FFFF0000,BASS_MUSIC_LOOP+BASS_MUSIC_FT2MOD,0) then
		end if
	end if
	
	c_proc(glutReshapeFunc,{hiscore_reshape_cb})
	c_proc(glutDisplayFunc,{hiscore_display_cb})
	c_proc(glutSpecialFunc,{0})
	c_proc(glutKeyboardFunc,{hiscore_input_cb})
	joyhandler_id = hjoy_hiscore_id
	
	i = hiscore_reshape(game_window[1],game_window[2])	-- Make sure the reshape is done
end procedure


-----------------------------------------------------------------------------------------------


global procedure hiscore_get_name_setup()
	atom an,delta_an

	name_chars_length = length(hiscore_name_chars)
	an = 0.0
	delta_an = 6.2832/name_chars_length	-- 6.2832 = ~PI*2

	cos_table = repeat(0.0,name_chars_length)
	sine_table = repeat(0.0,name_chars_length)
	sine_table2 = repeat(0.0,name_chars_length)

	for i=1 to name_chars_length do
		sine_table[i] = sin(an-1.5708)
		sine_table2[i] = sin(an)
		cos_table[i] = cos(an-1.5708)
		an += delta_an
	end for
end procedure


function hiscore_get_name_display()
	integer c,cur_char
	atom s,t,x,y,col_factor
	atom an,delta_an

	c_proc(glClear,{GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT})
	c_proc(glEnable,{GL_TEXTURE_2D})

	c_proc(glColor4f,{1,1,1,1})

	c_proc(glBindTexture,{GL_TEXTURE_2D,pane})
	c_proc(glBegin,{GL_QUADS})
		c_proc(glTexCoord2f,{0,0})
		c_proc(glVertex3f,{-1,-0.2,-1.1})
		c_proc(glTexCoord2f,{0,1})
		c_proc(glVertex3f,{-1,-1,-1.1})
		c_proc(glTexCoord2f,{1,1})
		c_proc(glVertex3f,{-0.3,-1,-1.1})
		c_proc(glTexCoord2f,{1,0})
		c_proc(glVertex3f,{-0.3,-0.2,-1.1})

		c_proc(glTexCoord2f,{1,1})
		c_proc(glVertex3f,{0.3,1,-1.1})
		c_proc(glTexCoord2f,{1,0})
		c_proc(glVertex3f,{0.3,0.2,-1.1})
		c_proc(glTexCoord2f,{0,0})
		c_proc(glVertex3f,{1,0.2,-1.1})
		c_proc(glTexCoord2f,{0,1})
		c_proc(glVertex3f,{1,1,-1.1})
	c_proc(glEnd,{})

	-- Draw some informative text
	c_proc(glColor3f,{1,0.9,0.7})
	putsxyz("Please enter your name into",font1,{-0.95,0.85,-1.1},16,{0.075,0.075},0.07)
	putsxyz("the Tetrix3 hall of fame!",font1,{-0.95,0.75,-1.1},16,{0.075,0.075},0.07)

	c_proc(glBindTexture,{GL_TEXTURE_2D,font4})
	c_proc(glEnable,{GL_BLEND})
	c_proc(glBlendFunc,{GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA})
	c_proc(glDisable,{GL_DEPTH_TEST})

	-- Form an ellipse of all the allowed characters
	cur_char = name_char
	delta_an = 360.0/name_chars_length	
	an = 0.0
	for i=1 to name_chars_length do
		c = hiscore_name_chars[cur_char]
		if (c != ' ') then
			c -= ' '
			-- Calculate the texture coordinates
			s = and_bits(c,#0F)*16
			t = and_bits(c,#F0)
			s /= 256.0
			t /= 256.0

			x = cos_table[i]*X_RADIUS
			y = sine_table[i]*Y_RADIUS
			col_factor = 1-((y+0.35)/0.7)
			c_proc(glColor3f,{0.5+col_factor*0.5,
					  0.3+col_factor*0.6,
					  0.1+col_factor*0.25})
			c_proc(glPushMatrix,{})
			c_proc(glTranslatef,{x, 0.2+y*0.5, -y-1.51})
			c_proc(glRotatef,{an,0,1,0})

			c_proc(glBegin,{GL_QUADS})
				c_proc(glTexCoord2f,{s, t})
				c_proc(glVertex3f,{-0.06,0.06, 0})

				c_proc(glTexCoord2f,{s, t+0.05859375})	-- 0.05859375 = 15/256
				c_proc(glVertex3f,{-0.06, -0.06, 0})

				c_proc(glTexCoord2f,{s + 0.05859375, t + 0.05859375})
				c_proc(glVertex3f,{0.06,-0.06, 0})

				c_proc(glTexCoord2f,{s + 0.05859375, t})
				c_proc(glVertex3f,{0.06, 0.06, 0})
			c_proc(glEnd,{})
			c_proc(glPopMatrix,{})
		end if
		an += delta_an
		cur_char += 1
		if (cur_char > name_chars_length) then
			cur_char = 1
		end if
	end for

	-- Draw the name in large blue letters
	c_proc(glColor3f,{0.5,0.4,1})
	putsxyz(hiscore_name,font4,{-0.36,-0.45,-1.1},16,{0.28,0.18},0.24)

	-- Draw a blinking marker
	c_proc(glDisable,{GL_TEXTURE_2D})
	c_proc(glDisable,{GL_BLEND})
	c_proc(glPolygonMode,{GL_FRONT_AND_BACK,GL_LINE})
	c_proc(glLineWidth,{4})
	c_proc(glColor3f,hiscore_pal[hiscore_col+1])
	c_proc(glBegin,{GL_QUADS})
		c_proc(glVertex3f,{-0.07,0.07,-1.11})
		c_proc(glVertex3f,{-0.07,-0.08,-1.11})
		c_proc(glVertex3f,{0.07,-0.08,-1.11})
		c_proc(glVertex3f,{0.07,0.07,-1.11})
	c_proc(glEnd,{})
	c_proc(glPolygonMode,{GL_FRONT_AND_BACK,GL_FILL})

	c_proc(glutSwapBuffers,{})

	hiscore_colchange -= 1
	if (hiscore_colchange=0) then
		hiscore_col += hiscore_dcol
		if ((hiscore_col=0) or (hiscore_col=11)) then
			hiscore_dcol = -hiscore_dcol
		end if
		hiscore_colchange = 2
	end if

	return 0
end function
hiscore_get_name_display_cb = call_back(routine_id("hiscore_get_name_display"))



function hiscore_get_name_reshape(integer w, integer h)
	game_window = {w,h}
	c_proc(glViewport,{0,0,w,h})

	return 0
end function
hiscore_get_name_reshape_cb = call_back(routine_id("hiscore_get_name_reshape"))



procedure hjoy_hiscore_get_name()
	if joybtn[1]=-1 then
		name_char -= 1
		if (name_char<1) then
			name_char = name_chars_length
		end if
	elsif joybtn[1]=1 then
		name_char += 1
		if (name_char>name_chars_length) then
			name_char = 1
		end if
	end if

	if joybtn[3] or joybtn[4] then
		if name_char=41 then
			-- END
			--songs[4] = music_stop(songs[4])
			call_proc(hiscore_update_list_id,{})
			hiscore_prepare()
		elsif name_char=40 then
			-- ERASE
			if (name_index>1) then
				name_index -= 1
				hiscore_name[name_index] = ' '
			end if
		else
			if (name_index<4) then
				hiscore_name[name_index] = hiscore_name_chars[name_char]
				name_index += 1
				if (name_index=4) then
					name_char = name_chars_length
				end if
			end if
		end if
	end if
end procedure
constant hjoy_hiscore_get_name_id = routine_id("hjoy_hiscore_get_name")



function hiscore_get_name_input(integer key_code,atom x,atom y)
	key_code = and_bits(key_code,#FF)	-- BUGFIX 2003-06-20
	
	if (key_code=13) then
		if name_char=41 then
			-- END
			--songs[4] = music_stop(songs[4])
			call_proc(hiscore_update_list_id,{})
			hiscore_prepare()
		elsif name_char=40 then
			-- ERASE
			if (name_index>1) then
				name_index -= 1
				hiscore_name[name_index] = ' '
			end if
		else
			if (name_index<4) then
				hiscore_name[name_index] = hiscore_name_chars[name_char]
				name_index += 1
				if (name_index=4) then
					name_char = name_chars_length
				end if
			end if
		end if
	end if
	
	return 0
end function
hiscore_get_name_input_cb = call_back(routine_id("hiscore_get_name_input"))



function hiscore_get_name_spec_input(integer key_code,atom x,atom y)
	if key_code=GLUT_KEY_RIGHT then
		name_char += 1
		if (name_char>name_chars_length) then
			name_char = 1
		end if
	elsif key_code=GLUT_KEY_LEFT then
		name_char -= 1
		if (name_char<1) then
			name_char = name_chars_length
		end if
	end if
	
	return 0
end function
hiscore_get_name_spec_input_cb = call_back(routine_id("hiscore_get_name_spec_input"))



global procedure hiscore_get_name()
	integer i

	c_proc(glMatrixMode,{GL_PROJECTION})
	c_proc(glLoadIdentity,{})
	c_proc(glFrustum,{-1, 1, -1, 1, 1.1, 20})
	glu_lookAt({0, 0, 0},	-- Pos. of camera
	      	   {0, 0, -5},	-- Look at
	    	   {0, 1, 0})	-- Normalvector
	c_proc(glMatrixMode,{GL_MODELVIEW})
	c_proc(glLoadIdentity,{})
	c_proc(glHint,{GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST})

	hiscore_name = "   "
	name_index = 1
	name_char = 1

	hiscore_col = 0
	hiscore_colchange = 2
	hiscore_dcol = 1

	if not PlayModEx(songs[5],#FFFF0000,BASS_MUSIC_LOOP+BASS_MUSIC_FT2MOD,0) then
	end if
	hiscore_music_playing = 1
	
	c_proc(glutReshapeFunc,{hiscore_get_name_reshape_cb})
	c_proc(glutDisplayFunc,{hiscore_get_name_display_cb})
	c_proc(glutSpecialFunc,{hiscore_get_name_spec_input_cb})
	c_proc(glutKeyboardFunc,{hiscore_get_name_input_cb})
	joyhandler_id = hjoy_hiscore_get_name_id
	i = hiscore_get_name_reshape(game_window[1],game_window[2]) -- Make sure the reshape is done
end procedure


-----------------------------------------------------------------------------------------------


-- Returns the position if we made it into the top 5, otherwise -1
global function hiscore_check()
	integer j
	
	j = -1
	for i=1 to 5 do
		if (score > entries[i][2]) then
			j = i
			exit
		end if
	end for
	
	return j
end function



-- Update the highscore list
global procedure hiscore_update_list()
	integer n
	
	n = 0
	for i=1 to 5 do
		if (score > entries[i][2]) then
			n = i
			exit
		end if
	end for

	latest_entry = n
	for j=5 to n+1 by -1 do
		entries[j] = entries[j-1]
	end for
	entries[latest_entry][1] = hiscore_name
	entries[latest_entry][2] = score
	entries[latest_entry][3] = sprintf("%06d",score)

	hiscore_file = open("hiscore","wb")
	if hiscore_file=-1 then
		puts(1,"Failed to open highscore file!")
		abort(0)
	end if
	for j=1 to 5 do
		puts(hiscore_file,entries[j][1])
		puts(hiscore_file,int_to_bytes(entries[j][2]))
	end for
	close(hiscore_file)
end procedure
hiscore_update_list_id = routine_id("hiscore_update_list")



