include titlescreen.e
include piece.e
include utils.e
include options.e
include joy.ew

--with profile
--profile(1)

constant
	level_prerequisites = {15,30,50,70,95,120,150,200,300},
	sample_files = {
	{
		"tmove.wav",
		"tflip.wav",
		"thit.wav",
		"tdrop.wav",
		"tscore.wav",
		"abunai.wav",
		"beautiful.wav",
		"byebye.wav",
		"huhu.wav",
		"lakki.wav",
		"lakkideshou.wav",
		"nicedesune.wav",
		"ok.wav",
		"wow.wav",
		"yottone.wav"
	},
	{	-- Initial volume
		70,
		50,
		65,
		60,
		65,
		31,
		37,
		28,
		38,
		40,
		38,
		35,
		40,
		35,
		45
	}}		

atom tetrixWindow

sequence
	cam1_pos,
	cam2_up,
	c1_angle,
	c1_d_angle,
	c2_angle,
	c2_d_angle,
	cam_inv_dir,
	score_string,
	lines_string,
	level_string,
	song_tempo
	


procedure cam_init()
	cam1_pos = {0,0,0}
	c1_angle = {90,0}
	c1_d_angle = {0.25,1}

	c2_angle = {90,0}
	c2_d_angle = {0.25,0}
	
	cam_inv_dir = {1,1}
end procedure



procedure cam_update()
	atom lev
	sequence rad
	atom an_diff
	
	lev = (level-1)*0.5
	an_diff = lev*7.0
	rad = c1_angle*0.01745

	cam1_pos = {cos(rad[1])*3, 0, cam1_pos[3]}
	cam1_pos[3] = sin(rad[2])*lev*0.6 + (lev*0.5)
	
	c1_angle += {c1_d_angle[1]*lev,c1_d_angle[2]}*speed_factor
	if (c1_angle[1]<90.0-an_diff) or (c1_angle[1]>90.0+an_diff) then
		if cam_inv_dir[1] then
			c1_d_angle[1] = -c1_d_angle[1]
			cam_inv_dir[1] = 0
		end if
	else
		cam_inv_dir[1] = 1
	end if
	if (c1_angle[2] >= 360.0) then
		c1_angle[2] -= 360.0
	end if

	an_diff = lev*8.0
	rad = c2_angle*0.01745

	cam2_up = {cos(rad[1]),sin(rad[1]),0} --sin(rad)}
	c2_angle += c2_d_angle*lev*speed_factor
	if ((c2_angle[1]<90.0-an_diff) or (c2_angle[1]>90.0+an_diff)) then
		if cam_inv_dir[2] then
			c2_d_angle[1] = -c2_d_angle[1]
			cam_inv_dir[2] = 0
		end if
	else
		cam_inv_dir[2] = 1
	end if
end procedure



-- Check if there are complete rows, and if so remove them
function check_complete()
	integer k,rows

	score += 5*(level+1)

	rows = 0

	-- Check for complete rows
	for i=1 to 18 do
		k = 0
		for j=1 to 10 do
			if (field[i][j][B_TYPE]) then
				k += 1
			end if
		end for

		if (k=10) then
			rows += 1
			for x=1 to 10 do
				if field[i][x][B_TYPE]<16 then
					field[i][x][B_TYPE] += 16
				end if
				field[i][x] = setup_exploding_block(field[i][x],x,i)
			end for
		end if

	end for

	lines += rows
	for i=1 to 9 do 
		if ((lines>=level_prerequisites[i]) and level=i) then
			-- We're clear for the next level
			level = i+1
			level_factor += 0.3
			SetFrequency(songs[game_music],
					song_tempo[game_music]+floor(song_tempo[game_music]*level/12))
			k = rand(5)
			if k<2 then
				sample_play(samples[SAMPLE_ABUNAI])
			elsif k=3 then
				sample_play(samples[SAMPLE_BEAUTIFUL])
			else
				sample_play(samples[SAMPLE_NICEDESUNE])
			end if	
			exit
		end if
	end for
	
	if rows=1 then
		score += 100+(level*50)
		if stack_height>13 then
			k = rand(3)
			if k=1 then
				sample_play(samples[SAMPLE_NICEDESUNE])
			elsif k=2 then
				sample_play(samples[SAMPLE_LAKKI])
			end if			
		else
			if rand(5)=4 then
				sample_play(samples[SAMPLE_OK])
			end if
		end if
	elsif rows=2 then
		score += 250+(level*125)
		k = rand(9)
		if stack_height>12 then
			if k<5 then
				sample_play(samples[SAMPLE_LAKKIDESHOU])
			else
				sample_play(samples[SAMPLE_WOW])
			end if	
		else
			if k<2 then
				sample_play(samples[SAMPLE_LAKKI])
			elsif k=3 then
				sample_play(samples[SAMPLE_OK])
			elsif k=7 then
				sample_play(samples[SAMPLE_NICEDESUNE])
			end if	
		end if
		
	elsif rows=3 then
		score += 500+(level*250)
		k = rand(5)
		if stack_height>11 then
			if k<4 then
				sample_play(samples[SAMPLE_LAKKI])
			else
				sample_play(samples[SAMPLE_YOTTONE])
			end if	
		
		else
			if k=1 or k=4 then
				sample_play(samples[SAMPLE_NICEDESUNE])
			elsif k=3 then
				sample_play(samples[SAMPLE_WOW])
			end if		
		end if
	elsif rows=4 then
		score += 1000*(level+1)
		k = rand(11)
		if stack_height>12 then
			if k<4 then
				sample_play(samples[SAMPLE_LAKKIDESHOU])
			elsif k>5 and k<9 then
				sample_play(samples[SAMPLE_BEAUTIFUL])
			else
				sample_play(samples[SAMPLE_YOTTONE])
			end if
		else
			if k<2 then
				sample_play(samples[SAMPLE_LAKKI])
			elsif k=5 or k=6 then
				sample_play(samples[SAMPLE_YOTTONE])
			elsif k=7 then
				sample_play(samples[SAMPLE_WOW])
			elsif k=10 then
				sample_play(samples[SAMPLE_BEAUTIFUL])
			end if	
		end if
	end if
	if (score > 999999) then
		score = 999999
	end if
	
	stack_height -= rows
	
	score_string = sprintf("%06d",score)
	lines_string = sprintf("%d",lines)
	level_string = sprintf("%d",level)

	if rows then
		sample_play(samples[SAMPLE_SCORE])
	end if
	return (rows>0)
end function
check_complete_id = routine_id("check_complete")



procedure remove_rows()
	integer contact,y
	
	contact = 1

	y = 1
	for i=18 to 1 by -1 do
		if (field[i][1][B_TYPE]>16) then
			contact = 0
			y = i
			exit
		end if
	end for
	if (not contact) then
		for x=1 to 10 do
			field[1][x][B_TYPE] = 0
		end for
	end if

	if (y=1) then
		return
	else
		while (not contact) do
			contact = 0
			for i=y to 2 by -1 do
				for x=1 to 10 do
					field[i][x] = field[i-1][x]
					contact = contact or ((field[i][x][B_TYPE]!=0) and
					                      (field[i][x][B_TYPE]<17))
				end for
			end for
			for x=1 to 10 do
				field[1][x][B_TYPE] = 0
			end for
		end while
	end if
end procedure



procedure select_material(sequence matamb,sequence matdiff,sequence matspec,integer mat)
	gl_materialfv(GL_FRONT_AND_BACK, GL_AMBIENT, matamb[mat])
	gl_materialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, matdiff[mat])
	gl_materialfv(GL_FRONT_AND_BACK, GL_SPECULAR, matspec[mat])
end procedure



procedure draw_block_basic(integer x, atom y, integer color)
  	c_proc(glPushMatrix,{})
  	c_proc(glTranslatef,{-6.0+x*1.2,9.0-y,0})
  	
  	c_proc(glCallList,{display_lists})
  	
  	c_proc(glPopMatrix,{})
  end procedure
  
  
procedure draw_block_basic_old(integer x, atom y, integer color)
  	c_proc(glPushMatrix,{})
  	c_proc(glTranslatef,{-6.0+x*1.2,9.0-y,0})

  	c_proc(glBegin,{GL_QUADS})
		-- Front
		c_proc(glVertex3f,vertexes[1])
		c_proc(glVertex3f,vertexes[2])
		c_proc(glVertex3f,vertexes[3])
		c_proc(glVertex3f,vertexes[4])

		-- Left
		c_proc(glVertex3f,vertexes[4])
		c_proc(glVertex3f,vertexes[3])
		c_proc(glVertex3f,vertexes[6])
		c_proc(glVertex3f,vertexes[5])

		-- Back
		c_proc(glVertex3f,vertexes[5])
		c_proc(glVertex3f,vertexes[6])
		c_proc(glVertex3f,vertexes[7])
		c_proc(glVertex3f,vertexes[8])

		-- Right
		c_proc(glVertex3f,vertexes[8])
		c_proc(glVertex3f,vertexes[7])
		c_proc(glVertex3f,vertexes[2])
		c_proc(glVertex3f,vertexes[1])

		-- Top
		c_proc(glVertex3f,vertexes[3])
		c_proc(glVertex3f,vertexes[2])
		c_proc(glVertex3f,vertexes[7])
		c_proc(glVertex3f,vertexes[6])

		-- Bottom
		c_proc(glVertex3f,vertexes[5])
		c_proc(glVertex3f,vertexes[8])
		c_proc(glVertex3f,vertexes[1])
		c_proc(glVertex3f,vertexes[4])

  	c_proc(glEnd,{})
  	c_proc(glPopMatrix,{})
end procedure



procedure draw_block_smooth(integer x, atom y, sequence block)
	integer iy
	
	iy = floor(y)
	if iy<0 then
		iy = 0
	end if
	
  	c_proc(glPushMatrix,{})
  	c_proc(glTranslatef,{-6.0+x*1.2,9.0-y,0})

  	c_proc(glBegin,{GL_QUADS})
		-- Front
		c_proc(glNormal3f,vertex_normals[iy+1][x+1][1])
		c_proc(glVertex3f,vertexes[1])
		c_proc(glNormal3f,vertex_normals[iy+2][x+1][1])
		c_proc(glVertex3f,vertexes[2])
		c_proc(glNormal3f,vertex_normals[iy+1][x+1+1][1])
		c_proc(glVertex3f,vertexes[3])
		c_proc(glNormal3f,vertex_normals[iy+2][x+1+1][1])
		c_proc(glVertex3f,vertexes[4])

		-- Left
		c_proc(glNormal3f,vertex_normals[iy+2][x+1+1][1])
		c_proc(glVertex3f,vertexes[4])
		c_proc(glNormal3f,vertex_normals[iy+1][x+1+1][1])
		c_proc(glVertex3f,vertexes[3])
		c_proc(glNormal3f,vertex_normals[iy+1][x+1+1][2])
		c_proc(glVertex3f,vertexes[6])
		c_proc(glNormal3f,vertex_normals[iy+2][x+1+1][2])
		c_proc(glVertex3f,vertexes[5])

		-- Back
		c_proc(glVertex3f,vertexes[5])
		c_proc(glVertex3f,vertexes[6])
		c_proc(glVertex3f,vertexes[7])
		c_proc(glVertex3f,vertexes[8])

		-- Right
		c_proc(glNormal3f,vertex_normals[iy+1][x+1][2])
		c_proc(glVertex3f,vertexes[8])
		c_proc(glNormal3f,vertex_normals[iy+2][x+1][2])
		c_proc(glVertex3f,vertexes[7])
		c_proc(glNormal3f,vertex_normals[iy+2][x+1][1])
		c_proc(glVertex3f,vertexes[2])
		c_proc(glNormal3f,vertex_normals[iy+1][x+1][1])
		c_proc(glVertex3f,vertexes[1])

		-- Top
		c_proc(glNormal3f,vertex_normals[iy+1][x+1+1][1])
		c_proc(glVertex3f,vertexes[3])
		c_proc(glNormal3f,vertex_normals[iy+2][x+1][1])
		c_proc(glVertex3f,vertexes[2])
		c_proc(glNormal3f,vertex_normals[iy+2][x+1][2])
		c_proc(glVertex3f,vertexes[7])
		c_proc(glNormal3f,vertex_normals[iy+1][x+1+1][2])
		c_proc(glVertex3f,vertexes[6])

		-- Bottom
		c_proc(glVertex3f,vertexes[5])
		c_proc(glVertex3f,vertexes[8])
		c_proc(glVertex3f,vertexes[1])
		c_proc(glVertex3f,vertexes[4])

  	c_proc(glEnd,{})
  	c_proc(glPopMatrix,{})
end procedure



procedure draw_row(integer y)
	sequence r
	integer color
	
	r = field[y]
	if rendering_mode=NO_SHADING then
		c_proc(glPushMatrix,{})
		c_proc(glTranslatef,{-6.0,10.0-y,0.0})
		
		for x=0 to 9 do
			color = r[x+1][B_TYPE]
			if color>0 and color<8 then
				c_proc(glEnable,{GL_CULL_FACE})	
				c_proc(glColor4f,possible_colors[color])
  	  			c_proc(glCallList,{display_lists})

				c_proc(glDisable,{GL_CULL_FACE})
				c_proc(glPolygonMode,{GL_FRONT_AND_BACK, GL_LINE})
				c_proc(glDepthFunc,{GL_LEQUAL})
				c_proc(glLineWidth,{2.0})
				c_proc(glColor4f,possible_colors2[color])
  	  			c_proc(glCallList,{display_lists})
				c_proc(glDepthFunc,{GL_LESS})
				c_proc(glPolygonMode,{GL_FRONT_AND_BACK, GL_FILL})
			end if
			c_proc(glTranslatef,{1.2,0.0,0.0})
		end for
		c_proc(glDisable,{GL_CULL_FACE})
		c_proc(glPopMatrix,{})		
	end if
end procedure



procedure draw_block(integer x, atom y, sequence block, integer rend_type)
	integer color
	
	color = block[B_TYPE]

	if (color=0 or color>7) then return end if
	--color--;

	-- The different rendering paths..
	if rend_type=NO_SHADING or rend_type!=SMOOTH_SHADING then
		c_proc(glPolygonMode,{GL_FRONT_AND_BACK, GL_FILL})
		c_proc(glColor4f,possible_colors[color])
		draw_block_basic(x, y, color)
		c_proc(glPolygonMode,{GL_FRONT_AND_BACK, GL_LINE})
		--c_proc(glCullFace,{GL_FRONT})
		c_proc(glDepthFunc,{GL_LEQUAL})
		c_proc(glLineWidth,{2.0})
		c_proc(glColor4f,possible_colors[color]/2)
		draw_block_basic(x, y, color)
		c_proc(glDepthFunc,{GL_LESS})
		c_proc(glCullFace,{GL_BACK})
		c_proc(glPolygonMode,{GL_FRONT_AND_BACK, GL_FILL})
	
	elsif rend_type=SMOOTH_SHADING then
		c_proc(glShadeModel,{GL_SMOOTH})
		c_proc(glEnable,{GL_DEPTH_TEST})
		c_proc(glEnable,{GL_CULL_FACE})
		c_proc(glCullFace,{GL_BACK})
		c_proc(glEnable,{GL_LIGHTING})
		c_proc(glEnable,{GL_LIGHT0})
		select_material(possible_colors_amb,
			        possible_colors,
				possible_colors_spec,
				color)
		draw_block_smooth(x, y, block)
		c_proc(glDisable,{GL_LIGHTING})
		c_proc(glPolygonMode,{GL_BACK, GL_LINE})
		c_proc(glCullFace,{GL_FRONT})
		c_proc(glDepthFunc,{GL_LEQUAL})
		c_proc(glLineWidth,{2.0})
		c_proc(glColor4f,possible_colors[color]/2)
		draw_block_basic(x, y, color)
		c_proc(glDisable,{GL_CULL_FACE})
		c_proc(glDepthFunc,{GL_LESS})
		c_proc(glCullFace,{GL_BACK})
		c_proc(glPolygonMode,{GL_FRONT_AND_BACK, GL_FILL})
	end if
end procedure
draw_block_id = routine_id("draw_block")



procedure draw_field()
	integer remove
	sequence res

			
	c_proc(glColor4f,{0.7, 1, 0.9, 0.8})
	--c_proc(glEnable,{GL_BLEND})
	--c_proc(glBlendFunc,{GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA})
	
	c_proc(glBegin,{GL_QUADS})

	-- Left side
	c_proc(glVertex3f,{-6.5, 9.5, 1})
	c_proc(glVertex3f,{-6.5, 9.5, -1})
	c_proc(glVertex3f,{-6.5, -8.5, -1})
	c_proc(glVertex3f,{-6.5, -8.5, 1})

	-- Bottom
	c_proc(glVertex3f,{-6.5, -8.5, -1})
	c_proc(glVertex3f,{5.5, -8.5, -1})
	c_proc(glVertex3f,{5.5, -8.5, 1})
	c_proc(glVertex3f,{-6.5, -8.5, 1})

	-- Right side
	c_proc(glVertex3f,{5.5, 9.5, -1})
	c_proc(glVertex3f,{5.5, 9.5, 1})
	c_proc(glVertex3f,{5.5, -8.5, 1})
	c_proc(glVertex3f,{5.5, -8.5, -1})

	c_proc(glEnd,{})

	--c_proc(glDisable,{GL_BLEND})

	remove = 0
	no_explosions = 1
	for y = 18 to 1 by -1 do
		if (field[y][1][B_TYPE]>16) then
			remove = 1
			res = draw_exploding_blocks(field[y])
			field[y] = res[2]
			no_explosions = no_explosions and res[1]
		else
			--for x=1 to 10 do
			--	draw_block(x-1, y-1, field[y][x], rendering_mode)
			--end for
			draw_row(y)
		end if
	end for

	if (remove and no_explosions and not end_game) then
		sample_play(samples[SAMPLE_DROP])
		remove_rows()
		cur_piece = init_piece(cur_piece)
	end if
end procedure



function display()
	integer screen_split,c
	sequence bmNext
	atom m,n,o,t
	
	-- Cheap pause function
	if (paused) then 
		return 0
	end if

	t = time()
	speed_factor = (t-last_frame_time)/0.01667
	last_frame_time = t
	
	screen_split = floor((game_window[1]*3)/4)

	-- Set the first viewport (the main window)
	c_proc(glViewport,{0,0,screen_split,game_window[2]})
	c_proc(glClear,{GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT})

	c_proc(glPushMatrix,{})
	c_proc(glTranslatef,{0, 0, -12})

	c_proc(glMatrixMode,{GL_PROJECTION})
	c_proc(glLoadIdentity,{})
	c_proc(glFrustum,{-1, 1, -1, 1, 1.1, 20})
	c_proc(glPushMatrix,{})


	cam_update()
	c_proc(gluLookAt,cam1_pos&{0,0,-5}&cam2_up)	
	    		
	-- Draw all the pieces etc.
	draw_field()
    	if (no_explosions) then
		cur_piece = draw_piece(cur_piece)
	end if	
	
	c_proc(glPopMatrix,{})
	c_proc(gluLookAt,{0, 0, 0,	-- Pos. of camera
	    	  	  0, 0, -1,	-- Look at
	    	  	  0, 1, 0}) 	-- Normalvector
	c_proc(glMatrixMode,{GL_MODELVIEW})

	c_proc(glPopMatrix,{})
	c_proc(glPushMatrix,{})
	c_proc(glTranslatef,{0,0,-1.1})


	-- Set the second viewport (the status window)
	c_proc(glViewport,{screen_split,0,floor(game_window[1]/4),game_window[2]})
	c_proc(glColor3f,{0.2,0.3,0.7})
	c_proc(glBegin,{GL_LINES})
		c_proc(glVertex3f,{-1,1.1,-0.1})
		c_proc(glVertex3f,{-1,-1.1,-0.1})
	c_proc(glEnd,{})
	
	c_proc(glColor3f,{0.9,0.9,1.0})
	putsxyz("Next:", font1,	{-0.75,0.88,-0.01}, 16, {0.24,0.12}, 0.18)

	c_proc(glColor3f,{0.9,0.9,1.0})
	putsxyz("Score:", font1, {-0.75,0.3,-0.01}, 16, {0.24,0.12}, 0.18)
	c_proc(glColor3f,{1.0,0.9,0})
	putsxyz(score_string, font1, {-0.75,0.16,-0.01}, 16, {0.24,0.12}, 0.18)

	c_proc(glColor3f,{0.9,0.9,1.0})
	putsxyz("Lines:", font1, {-0.75,-0.1,-0.01}, 16, {0.24,0.12}, 0.18)
	c_proc(glColor3f,{1.0,0.9,0})
	putsxyz(lines_string, font1, {-0.21,-0.24,-0.01}, 16, {0.24,0.12}, 0.18)

	c_proc(glColor3f,{0.9,0.9,1.0})
	putsxyz("Level:", font1, {-0.75,-0.5,-0.01}, 16, {0.24,0.12}, 0.18)
	c_proc(glColor3f,{1.0,0.9,0})
	putsxyz(level_string, font1, {-0.21,-0.64,-0.01}, 16, {0.24,0.12}, 0.18)

	c_proc(glDisable,{GL_TEXTURE_2D})

	-- Draw the next piece in the status window
	c_proc(glColor4f,possible_colors[next_piece])
	bmNext = pieces[next_piece][1]
	for i=1 to 4 do
		m = 0.71 - (i-1)*0.09
		n = 0.626 - (i-1)*0.09
		for j=1 to 4 do
			if (bmNext[i][j]) then
				o = (j-1)*0.329
				c_proc(glBegin,{GL_QUADS})
				c_proc(glVertex3f,{-0.43 + o, m, -0.01})
				c_proc(glVertex3f,{-0.43 + o, n, -0.01})
				c_proc(glVertex3f,{-0.125 + o, n, -0.01})
				c_proc(glVertex3f,{-0.125 + o, m, -0.01})
				c_proc(glEnd,{})
			end if
		end for
	end for
	c_proc(glColor4f,{1.0,1.0,1.0,1.0})

	c_proc(glPopMatrix,{})
	c_proc(glutSwapBuffers,{})

	return 0
end function
display_cb = call_back(routine_id("display"))



sequence jdelay
jdelay = {0,0,0,0,0,0,0}

procedure poll_joypad()
	sequence jstate
	
	jstate = joy_get_state(1)
	
	if not length(jstate) then	-- BUGFIX 2003-06-11
		has_joypad = 0
		return
	end if
	
	for i=1 to 4 do
		if jdelay[i] then
			jdelay[i] -= 1
		end if
	end for
	
	joybtn = {0,0,0,0,0}
	
	-- LEFT/RIGHT
	if jstate[1]=0 and not jdelay[1] then
		joybtn[1] = -1
		jdelay[1] = 4
	elsif jstate[1]=65535 and not jdelay[2] then
		joybtn[1] = 1
		jdelay[2] = 4
	end if
	
	-- UP/DOWN
	if jstate[2]=0 and not jdelay[3] then
		joybtn[2] = -1
		jdelay[3] = 3
	elsif jstate[2]=65535 and not jdelay[4] then
		joybtn[2] = 1
		jdelay[4] = 3
	end if
	
	if and_bits(jstate[4],1) and not jdelay[5] then
		joybtn[3] = 1
		jdelay[5] = 5
	elsif and_bits(jstate[4],2) and not jdelay[6] then
		joybtn[4] = 1
		jdelay[6] = 5
	elsif and_bits(jstate[4],64) and not jdelay[7] then
		joybtn[5] = 1
		jdelay[7] = 5
	end if

	if not and_bits(jstate[4],1) and jdelay[5] then
		jdelay[5] = 0
	end if
	if not and_bits(jstate[4],2) and jdelay[6] then
		jdelay[6] = 0
	end if
	if not and_bits(jstate[4],64) and jdelay[7] then
		jdelay[7] = 0
	end if
end procedure



function update(integer val)
	atom t,t2
	
	if val=1 then
		if has_joypad then
			poll_joypad()
			if joyhandler_id>0 then
				call_proc(joyhandler_id,{})
			end if
		end if
		--c_proc(glutPostRedisplay,{})
		--t = time()
		--t2 = 0.016-(t-last_frame_time)
		--if t2<0.005 then
		--	t2 = 0.005
		--end if
		--last_frame_time = t+t2
		--glut_timerFunc(floor(t2*1000), update_cb, 1)
	elsif val=2 then
		--songs[game_music] = music_stop(songs[game_music])
		StopMod(songs[game_music])
		hiscore_prepare()
	elsif val>=17 and val<=21 then
		StopMod(songs[game_music])
		--songs[game_music] = music_stop(songs[game_music])
		hiscore_get_name_setup()
		hiscore_get_name()
	end if
	return 0
end function
update_cb = call_back(routine_id("update"))


function idle()
	atom t,t2
	
	--t = time()
	--t2 = t-last_frame_time
	--if t2<=0 then
	--	speed_factor = 0.1
	--else
	--	speed_factor = t2/0.01667
	--end if
	--last_frame_time = t
	
	c_proc(glutPostRedisplay,{})
	return 0
end function
idle_cb = call_back(routine_id("idle"))


-------------------------------------------------------------------------------------------------------


procedure game_over()
	integer place
	
	if not end_game then
		end_game = 1

		-- Set all blocks to explode
		for i=1 to 18 do
			for j=1 to 10 do
				if field[i][j][B_TYPE]<16 then
					field[i][j][B_TYPE] += 16
				end if
				field[i][j] = setup_exploding_block(field[i][j],j,i)
			end for
		end for
	
		-- Check if the player has made it into the hiscore list
		place = hiscore_check()
		if (place != -1) then
			glut_timerFunc(1350,update_cb,place+16)
		else
			glut_timerFunc(1350,update_cb,2)
		end if
	end if
end procedure
game_over_id = routine_id("game_over")



procedure hjoy_main()
	if joybtn[1]=-1 then
		cur_piece = move_piece(cur_piece, LEFT, 0)
	elsif joybtn[1]=1 then
		cur_piece = move_piece(cur_piece, RIGHT, 0)
	elsif joybtn[2]=1 then
		cur_piece = move_piece(cur_piece, 0, 1)
	end if

	if joybtn[3] then
		cur_piece = rotate_piece(cur_piece,LEFT)
	
	elsif joybtn[4] then
		cur_piece = rotate_piece(cur_piece,RIGHT)
	elsif joybtn[5] then
		options_restore_displayfunc = display_cb
		options_restore_reshapefunc = reshape_cb
		options_restore_keyboardfunc = keyboard_cb
		options_restore_specfunc = spec_func_cb
		options_restore_joyhandler = joyhandler_id
		options()
	end if
end procedure
constant hjoy_main_id = routine_id("hjoy_main")



function spec_func(integer key_code, atom x, atom y)
  	if key_code=GLUT_KEY_F1 then
	  	c_proc(glutFullScreen,{})

	elsif key_code=GLUT_KEY_LEFT then
		cur_piece = move_piece(cur_piece, LEFT, 0)

	elsif key_code=GLUT_KEY_RIGHT then
		cur_piece = move_piece(cur_piece, RIGHT, 0)

	elsif key_code=GLUT_KEY_DOWN then
		cur_piece = move_piece(cur_piece, 0, 1)
	end if

	return 0
end function
spec_func_cb = call_back(routine_id("spec_func"))



function keyboard(integer key_code, atom x, atom y)
	key_code = and_bits(key_code,#FF)	-- BUGFIX 2003-06-20
	
	if key_code>='0' and key_code<='4' then		
		rendering_mode = key_code-'0'

	elsif key_code='z' then
		cur_piece = rotate_piece(cur_piece,LEFT)
	
	elsif key_code='x' then
		cur_piece = rotate_piece(cur_piece,RIGHT)

	elsif key_code='u' then
		song_tempo += 50
		--set_tempo("mid2",song_tempo)

	elsif key_code='d' then
		song_tempo -= 50
		--set_tempo("mid2",song_tempo)

	elsif key_code='+' then
		if (level<9) then
			level += 1
			level_factor += 0.3
			--song2->set_tempo(300+(level<<2))
		end if
		level_string = sprintf("%d",level)

	elsif key_code='-' then
		if (level>0) then
			level -= 1
			level_factor -= 0.3
			--song2->set_tempo(300+(level<<2))
		end if
		level_string = sprintf("%d",level)

	elsif key_code=13 then
		--paused = 1-paused

	elsif key_code=27 then	--ESC
		options_restore_displayfunc = display_cb
		options_restore_reshapefunc = reshape_cb
		options_restore_keyboardfunc = keyboard_cb
		options_restore_specfunc = spec_func_cb
		options_restore_joyhandler = hjoy_main_id
		options()
	end if
		
	return 0
end function
keyboard_cb = call_back(routine_id("keyboard"))



function reshape(integer w, integer h)
	game_window = {w,h}

	c_proc(glMatrixMode,{GL_MODELVIEW})
	c_proc(glLoadIdentity,{})

	return 0
end function
reshape_cb = call_back(routine_id("reshape"))


-------------------------------------------------------------------------------------------------------


procedure start_new_game()
	gl_enable(GL_DEPTH_TEST)
	c_proc(glClearColor,{0,0,0,0})
	
    	level_factor = 1.0
	score = 0
	lines = 0
	level = 1
	score_string = sprintf("%06d",score)
	lines_string = sprintf("%d",lines)
	level_string = sprintf("%d",level)

	no_explosions = 1
	end_game = 0
	paused = 0
	stack_height = 0
	
	light_direction = {0.2, 0.25, -1.0}
	light_direction = normalize(light_direction)

	field = repeat(repeat(new_block,10),18)
	for y=1 to 18 do
		for x=1 to 10 do
			field[y][x][B_TYPE] = 0
			field[y][x][B_BOOM] = repeat(new_particle,BOOM_SIZE)
		end for
	end for
	cur_piece = init_piece(cur_piece)
	cam_init()
	
	if music_enabled then
		game_music = 2 --rand(2)+1
		--music_set_volume(songs[game_music],#5000)
		--songs[game_music] = music_set_tempo(songs[game_music],song_tempo[game_music])
		--songs[game_music] = music_play(songs[game_music],250,7)
		--song2->set_tempo(300)
		SetFrequency(songs[game_music],0)
		if not PlayModEx(songs[game_music],#FFFF0000,BASS_MUSIC_LOOP+BASS_MUSIC_FT2MOD,0) then
		end if
	end if
	
	joyhandler_id = hjoy_main_id
	c_proc(glutReshapeFunc,{reshape_cb})
	c_proc(glutKeyboardFunc,{keyboard_cb})
	c_proc(glutSpecialFunc,{spec_func_cb})
	c_proc(glutDisplayFunc,{display_cb})

	if reshape(game_window[1],game_window[2]) then end if
end procedure
start_new_game_id = routine_id("start_new_game")



function dummy_display()
	c_proc(glClearColor,{0,0,0,0})
	c_proc(glClear,{GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT})
	
	return 0
end function
global constant dd_cb = call_back(routine_id("dummy_display"))


global sequence load1,load2,load3

function init_display()
	c_proc(glClear,{GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT})

	c_proc(glColor3f,{0.35,0.28,0.6})
	
	--putsxyz("Please enter your name into",font1,{-0.95,0.85,-1.1},16,{0.075,0.075},0.07)	
	putsxyz("========= TETRIX3 =========",font3,{-0.95,0.96,0},16,{0.075,0.075},0.07)	
	putsxyz("===========================",font3,{-0.95,-0.90,0},16,{0.075,0.075},0.07)
	if music_enabled then
		--putsxyz(midi_info[1],font3,{-0.95,0.7,0},16,{0.075,0.075},0.07)	
		--if length(midi_info[2]) then
		--	putsxyz(midi_info[2],font3,{-0.95,0.55,0},16,{0.075,0.075},0.07)	
		--	putsxyz(midi_info[3],font3,{-0.95,0.45,0},16,{0.075,0.075},0.07)	
		--end if
		putsxyz(load1,font3,{-0.95,0.7,0.1},16,{0.075,0.075},0.07)	
		putsxyz(load2,font3,{-0.95,0.55,0.1},16,{0.075,0.075},0.07)	
		putsxyz(load3,font3,{-0.95,0.4,0.1},16,{0.075,0.075},0.07)	
	else
		putsxyz(load3,font3,{-0.95,0.7,0.1},16,{0.075,0.075},0.07)	
	end if
	
	c_proc(glutSwapBuffers,{})
	
	return 0
end function
constant init_display_cb = call_back(routine_id("init_display"))


function init_reshape(integer w, integer h)
	-- Save the current width and height
	game_window = {w,h}
	
	-- Update the viewport
	gl_viewport(0,0,w,h)

	return 0
end function
constant init_reshape_cb = call_back(routine_id("init_reshape"))


constant resolutions = {{640,480},{800,600},{1024,768}}

function init(integer foo)
	sequence m

	rendering_mode = NO_SHADING

  	game_modes = {}
   	-- Check the supported game modes
   	for i=1 to length(resolutions) do
		for b=16 to 32 by 8 do
			m = sprintf("%dx%d:%d",resolutions[i]&b)
			glut_gameModeString(m)
			if c_func(glutGameModeGet,{GLUT_GAME_MODE_POSSIBLE}) then
				game_modes = append(game_modes,m)
			end if
		end for	
   	end for
   	
  	
	font3 = texture({"data\\font3.bmp",0,0,0,DISCARD_MEMORY})
	compile_lists()
	
	c_proc(glClearColor,{0.6,0.55,0.95,1.0})
	c_proc(glutDisplayFunc,{init_display_cb})
	c_proc(glutReshapeFunc,{init_reshape_cb})
	c_proc(glutIdleFunc,{idle_cb})

	cur_piece = new_piece()
	if init_display() then end if

	gl_lightfv(GL_LIGHT0,GL_POSITION,light_pos)
   	gl_lightfv(GL_LIGHT0,GL_SPOT_DIRECTION,light_dir)
 
   	
	if sound_enabled then
		if not InitSoundSystem(0,22050,16,1,0) then
			puts(1,"Failed to init sound system!")
			abort(0)
		end if
		SetMasterVolume(20)
	end if
	
	if music_enabled then
		load1 = "Loading music: "
		if init_display() then end if
		songs[1] = LoadMod("data\\song1.s3m",BASS_MUSIC_FT2MOD,1)
		SetVolume(songs[1],50)
		load1 &= '*'
		if init_display() then end if
		songs[2] = LoadMod("data\\song2.mod",BASS_MUSIC_FT2MOD+BASS_MUSIC_CALCLEN,1)
		SetVolume(songs[2],40)
		load1 &= '*'
		if init_display() then end if
		--songs[3] = music_load("data\\song3.mid")
		--load1 &= '*'
		--if init_display() then end if
		--songs[4] = LoadMod("data\\song4.mod")
		--load1 &= '*'
		--if init_display() then end if
		songs[5] = LoadMod("data\\song5.xm",BASS_MUSIC_FT2MOD+BASS_MUSIC_CALCLEN,1)
		SetVolume(songs[5],40)
		load1 &= '*'
		if init_display() then end if
		
		for i=1 to 5 do
			if atom(songs[i]) then
				song_tempo[i] = GetChannelAttributes(songs[i])
				if not length(song_tempo[i]) then
					puts(1,"Failed to get channel attributes!")
					abort(0)
				end if
				song_tempo[i] = song_tempo[i][1]
			end if
		end for
	end if
	
	if sound_enabled then
		load2 = "Loading samples: "
		if init_display() then end if
		for i=1 to length(sample_files[1]) do
			samples[i] = LoadWav("data\\"&sample_files[1][i],0,0)
			if not samples[i] then
				puts(1,"Failed to load sample: "&sample_files[1][i])
				abort(0)
			end if
			--SetVolume(samples[i],sample_files[2][i])
			samples[i] = {samples[i],sample_files[2][i]}
			if remainder(i,3)=0 then
				load2 &= '*'
				if init_display() then end if
			end if
		end for
	end if
	
	load3 = "Loading textures: "
	if init_display() then end if
	hiscore_setup()
	load3 &= '*'
	if init_display() then end if

	titlescreen_init()
	c_proc(glClearColor,{0,0,0,0})
	
	return 0
end function
global constant init_cb = call_back(routine_id("init"))


------------------------------------------------------------------------------------------------------


stack_height = 0
song_tempo = {0,0,0,0,0}
has_joypad = joy_init()
joybtn = {0,0,0,0,0}
joyhandler_id = 0
display_lists = -1
options_restore_reshape_id = 0
last_frame_time = time()


vertex_normals = repeat(repeat({0,0},11),19)
integer y1,y2,x1,x2
sequence v

for y=1 to 18 do
	y1 = y
	for x=1 to 10 do
		x1 = x
		for i=1 to 4 do
			if i=1 or i=4 then
				y2 = y1+1
			else
				y2 = y1
			end if
			if i<3 then
				x2 = x1+1
			else
				x2 = x1
			end if
			
			v = vertexes[i]
			for j=1 to 3 do
				v[j] /= fabs(v[j])
			end for
			v = normalize(v)
			v += {(x2-1)-4.5,9.0-(y2-1),0}
			v = normalize(v)
			vertex_normals[y2][x2][1] = v

			v = vertexes[9-i]
			for j=1 to 3 do
				v[j] /= fabs(v[j])
			end for
			v = normalize(v)
			v += {(x2-1)-4.5,9.0-(y2-1),0}
			v = normalize(v)
			vertex_normals[y2][x2][2] = v
		end for
	end for
end for

			