include globals.e
include block.e
include glue.e

--with profile
--profile(1)

constant
	P_ID		= 1,
	P_XPOS		= 2,
	P_YPOS		= 3,
	P_TYPE		= 4,
	P_ORIENTATION	= 5,
	P_SPEED		= 6,
	P_BITMAP	= 7
	


function set_piece_bitmap(sequence piece)
	piece[P_BITMAP] = pieces[piece[P_TYPE]][piece[P_ORIENTATION]]
	for i=1 to 4 do
		for j=1 to 4 do
			if piece[P_BITMAP][i][j] then
				piece[P_BITMAP][i][j] = block(piece[P_TYPE])
			else
				piece[P_BITMAP][i][j] = block(0)
			end if
		end for
	end for
	return piece
end function



global function new_piece()
	integer piece_type
	
	-- Type of piece
	piece_type = rand(7)
	-- Also generate the type of the next piece, so that we can display it.
	next_piece = rand(7)

	return set_piece_bitmap({1, 3, -4, piece_type, ORIENTATION_NORMAL, CONSTANT_SPEED*level_factor,{}})
end function




global function init_piece(sequence piece)
	piece[P_XPOS] = 3
	piece[P_YPOS] = -4
	
	-- Only randomize a new piece if we need one
	if (piece[P_TYPE]=-1) then
		piece[P_TYPE] = next_piece
		next_piece = rand(7)
	end if
	piece[P_ORIENTATION] = ORIENTATION_NORMAL
	piece = set_piece_bitmap(piece)
	piece[P_ID] += 1
	return piece
end function



function collision(sequence piece,integer movement)
	integer floor_y,round_y,x_pos
	atom speed,y_pos
	sequence bm,f

	floor_y = floor(piece[P_YPOS])
	if fabs(piece[P_YPOS] - floor_y) > 0.1 then
		round_y = floor_y+1
	else
		round_y = floor_y
	end if
	
	bm = piece[P_BITMAP]
	x_pos = piece[P_XPOS]
	y_pos = piece[P_YPOS]
	speed = piece[P_SPEED]
	

	for y=3 to 0 by -1 do
		for x=0 to 3 do
			if (bm[y+1][x+1][B_TYPE]) then
				-- Check bottom
				if (y_pos+y > 17) then
					return 1
				-- Check left border
				elsif (x_pos+x < 0) then
					return 1
				-- Check left border
				elsif (x_pos+x > 9) then
					return 1
				end if
				
				if (y_pos >= 0.0) and floor_y+y<17 then
					if (field[floor_y+y+1 +1][x_pos+x +1][B_TYPE]) then
						return 1
					end if
					if (floor_y + 1 - y_pos > speed*20) then
						if (field[floor_y+y +1][x_pos+x +1][B_TYPE]) then
							return 1
						end if
					end if
				elsif y_pos<0.0 then
					if (floor_y+y>=-1) then
						if (field[floor_y+y+2][x_pos+x +1][B_TYPE]) then
							return 1
						end if
					end if
				end if
			end if
		end for
	end for

	return 0
end function



global function rotate_piece(sequence piece,integer dir)
	integer old_orient
	sequence old_bm
	
	if piece[P_TYPE]=-1 or end_game then
		return piece
	end if
	
	old_orient = piece[P_ORIENTATION]
	old_bm = piece[P_BITMAP]
	
	if dir=LEFT then
		piece[P_ORIENTATION] += 1
		if piece[P_ORIENTATION]>4 then
			piece[P_ORIENTATION] = 1
		end if
	else
		piece[P_ORIENTATION] -= 1
		if piece[P_ORIENTATION]<1 then
			piece[P_ORIENTATION] = 4
		end if
	end if
	
	piece = set_piece_bitmap(piece)	-- BUGFIX 2003-06-11
	
	if collision(piece,2) then
		piece[P_ORIENTATION] = old_orient
		piece[P_BITMAP] = old_bm
	else
		sample_play(samples[SAMPLE_FLIP])
	end if
	
	return piece
end function



global function move_piece(sequence piece, integer x_dir, integer y_dir)
	integer y_temp
	atom y_old

	if end_game then
		return piece
	end if
	
	y_temp = floor(piece[P_YPOS])
	y_old = piece[P_YPOS]
	
	if (x_dir = LEFT) then
		piece[P_XPOS] -= 1
		if collision(piece,0) then
			piece[P_XPOS] += 1
		else
			sample_play(samples[SAMPLE_MOVE])
		end if
	elsif (x_dir = RIGHT) then
		piece[P_XPOS] += 1
		if collision(piece,0) then
			piece[P_XPOS] -= 1
		else
			sample_play(samples[SAMPLE_MOVE])
		end if
	end if
	
	if y_dir then
		piece[P_YPOS] += 1.0
		if collision(piece,1) then
			piece[P_YPOS] = y_temp+1
		end if
	end if

	return piece
end function




global function draw_piece(sequence piece)
	integer iy,x_pos
	integer collided
	integer last_sh
	
	if end_game then
		return piece
	end if
	
	collided = 0

	-- Draw the piece
	for i=0 to 3 do
		for j=0 to 3 do
			call_proc(draw_block_id,{piece[P_XPOS]+j,
						 piece[P_YPOS]+i,
						 piece[P_BITMAP][i+1][j+1],
						 rendering_mode})
		end for
	end for
	
	last_sh = stack_height
	
	piece[P_YPOS] += piece[P_SPEED]*speed_factor
	if collision(piece,1) then
		if (piece[P_YPOS]<0) then
			for i=1 to 4 do
				for j=1 to 4 do
					if piece[P_BITMAP][i][j][B_TYPE] then
						iy = floor(piece[P_YPOS]+i-1)
						exit
					end if
				end for
			end for
			if iy<0 then
				sample_play(samples[SAMPLE_HIT])
				call_proc(game_over_id,{})
				return piece
			end if
		end if

		iy = floor(piece[P_YPOS])
		x_pos = piece[P_XPOS]
		
		for i=1 to 4 do
			if iy+i>0 then
			--modelmat[2] = fx*1.2f - 6.0f;
			for j=1 to 4 do
				if (piece[P_BITMAP][i][j][B_TYPE]!=0) then
					if x_pos+j>0 and x_pos+j<11 then
						--? iy
						if 19-(iy+i) > stack_height then
							stack_height = 19-(iy+i)
						end if
						field[iy+i][x_pos+j][B_TYPE] = piece[P_BITMAP][i][j][B_TYPE]
						field[iy+i][x_pos+j][B_ID] = piece[P_ID]
						field[iy+i][x_pos+j][B_FLOATING] = 0
					end if
					
					--for (k=0; k<6; k++)
					--{
					--	vec_mul_mat((Vector3f *)&common_face_normals[k][0],
					--				&modelmat[0],
					--				&field[iy+i][x_pos+j].face_normals[k]);
					--}
					--calc_vertex_normals(&field[iy+i][x_pos+j]);
					--for (k=0; k<6; k++)
					--	normalize(&field[iy+i][x_pos+j].face_normals[k]);
				end if
				--modelmat[2] += 1.2f;
			end for
			end if
			--modelmat[5] -= 1.0f;
		end for
		--if iy=0 then
		--	? field[1]
		--end if

		piece[P_TYPE] = -1
		if (not call_func(check_complete_id,{})) then
			if stack_height-last_sh>=3 and last_sh>=3 then
				-- Ridicule the player for making such a bad move
				sample_play(samples[SAMPLE_HUHU])
			elsif stack_height>12 and rand(5)=2 then			
				-- Oh-oh! watch out!
				sample_play(samples[SAMPLE_ABUNAI])
			else			
				-- Normal hit sound
				sample_play(samples[SAMPLE_HIT])
			end if
			piece = init_piece(piece)
		end if
	else
		iy = floor(piece[P_YPOS])
		--modelmat[5] = 9.0f-y_pos;

		--for (i=0; i<4; i++)
		--{
		--	modelmat[2] = fx*1.2f - 6.0f;
		--	for (j=0; j<4; j++)
		--	{
		--		if (bitmap[i][j].type!=0)
		--		{
		--			for (k=0; k<6; k++)
		--			{
		--				vec_mul_mat((Vector3f *)&common_face_normals[k][0],
		--							&modelmat[0],
		--							&bitmap[i][j].face_normals[k]);
		--			}
		--			calc_vertex_normals(&bitmap[i][j]);
		--			for (k=0; k<6; k++)
		--				normalize(&bitmap[i][j].face_normals[k]);
		--		}
		--		modelmat[2] += 1.2f;
		--	}
		--	modelmat[5] -= 1.0f;
		--}
	end if


	-- Update the speed, in case we've moved up a level
	piece[P_SPEED] = CONSTANT_SPEED * level_factor
	return piece
end function




