include font.e
include sound.ew
include globals.e
include options.e
include utils.e


-- The grid of controlpoints for the morphing background
constant titlescreen_mesh = allocate(24*24*5*4)

atom
	phongmap,		-- Phong map (highlight)
        background,		-- Background texture
	title			-- Tetrix logo


------------------------------------------------------------------------------------------------------

atom
	s_trans,	-- Horizontal transformation-amount for background
	color_mult,
	fade_speed
	
integer choise		-- Current selection (see below string)		

constant choises = {"Start game","Options"}


------------------------------------------------------------------------------------------------------

integer hjoy_titlescreen_id
procedure hjoy_titlescreen()
	if joybtn[2]=-1 then
		if (choise>1) then choise-=1 end if
	elsif joybtn[2]=1 then
		if (choise<2) then choise+=1 end if
	end if

	if joybtn[3] or joybtn[4] then
		if ((choise=1) and (fade_speed=0)) then
			fade_speed = 0.015	-- Start the fadeout
		else
			-- Save the current callbacks
			options_restore_displayfunc = titlescreen_display_cb
			options_restore_reshapefunc = titlescreen_reshape_cb
			options_restore_keyboardfunc = titlescreen_input_cb
			options_restore_specfunc = titlescreen_spec_input_cb
			options_restore_joyhandler = hjoy_titlescreen_id
			-- Open the options-screen
			options()
		end if
	end if
end procedure
hjoy_titlescreen_id = routine_id("hjoy_titlescreen")


-- Catch special keypresses 
function titlescreen_spec_input(integer key_code, atom x, atom y)
	if key_code=GLUT_KEY_DOWN then	
		if (choise<2) then choise+=1 end if
	elsif key_code=GLUT_KEY_UP then
		if (choise>1) then choise-=1 end if
	elsif key_code=GLUT_KEY_F1 then
		glut_fullScreen()
	elsif key_code=13 then
		if ((choise=1) and (fade_speed=0)) then
			fade_speed = 0.015	-- Start the fadeout
		else
			-- Save the current callbacks
			options_restore_displayfunc = titlescreen_display_cb
			options_restore_reshapefunc = titlescreen_reshape_cb
			options_restore_keyboardfunc = titlescreen_input_cb
			options_restore_specfunc = titlescreen_spec_input_cb
			-- Open the options-screen
			options()
		end if
	end if

	return 0
end function
titlescreen_spec_input_cb = call_back(routine_id("titlescreen_spec_input"))


-- Keyboard callback
function titlescreen_input(integer key_code, atom x, atom y)
	atom t
	
	key_code = and_bits(key_code,#FF)	--BUGFIX 2003-07-03
	
	if key_code=13 or key_code='s' or key_code='S' then	-- Return-key
		if ((choise=1) and (fade_speed=0)) then
			fade_speed = 0.015	-- Start the fadeout
		--else
			StopMod(songs[1])
			sample_play(samples[SAMPLE_OK])
			t = time()
			while time()-t<0.777 do end while
			call_proc(start_new_game_id,{})
		else
			-- Save the current callbacks
			options_restore_displayfunc = titlescreen_display_cb
			options_restore_reshapefunc = titlescreen_reshape_cb
			options_restore_keyboardfunc = titlescreen_input_cb
			options_restore_specfunc = titlescreen_spec_input_cb
			-- Open the options-screen
			options()
		end if
	end if
	return 0
end function
titlescreen_input_cb = call_back(routine_id("titlescreen_input"))



function titlescreen_reshape(integer w, integer h)
	-- Save the current width and height
	game_window = {w,h}
	
	-- Update the viewport
	gl_viewport(0,0,w,h)

	return 0
end function
titlescreen_reshape_cb = call_back(routine_id("titlescreen_reshape"))



global procedure titlescreen_init()
	atom s,t,x,y
	atom v,dv
	sequence temp_mesh
	
	s_trans = 0.0	
	choise = 1			
	color_mult = 1.0
        fade_speed = 0.0
        
	temp_mesh = repeat(repeat(repeat(0,5),24),24)
	
	-- Load some textures
	phongmap = texture({"data\\phongmap.bmp",DISCARD_MEMORY})
	background = texture({"data\\bg2.bmp",DISCARD_MEMORY})
	-- Mark full white (#FFFFFF) as the transparent color for title.bmp
	title = texture({"data\\title.bmp",255,255,255,DISCARD_MEMORY})

	-- Load the font (Akbar and Arial)
	font1 = font("data\\font1.bmp")

	-- Initialize the background mesh
	-- Each point in the mesh (or grid) consists of five elements;
	--   a position in the world (x,y,z), and a texture coordinate (s,t).
	for i=0 to 11 do
		y = i
		v = 180.0 - (7.083*y)		-- angle (temp value, only for use in next row)
		dv = (2*(v-90.0))/24.0		-- angle delta
		v = 180.0 + (7.083*y)		-- real angle

		for j=0 to 23 do
			x = j

			-- Calculate the texture coordinates
			s = x / 24.0
			t = sin(v*3.14159/180.0)*0.212
			
			-- Groovy calculations...
			temp_mesh[i+1][j+1][1] = (2.2*x/23.0)-1.1
			temp_mesh[i+1][j+1][2] = 1.3-(2.6*y/23.0)
			temp_mesh[i+1][j+1][3] = -0.1-(11.5-fabs(j-11.5))*0.04*floor(i/2)
			temp_mesh[i+1][j+1][4] = s
			temp_mesh[i+1][j+1][5] = (y*0.063 + t)

			-- Mirror the upper part of the mesh
			temp_mesh[24-i][j+1][1] = temp_mesh[i+1][j+1][1]
			temp_mesh[24-i][j+1][2] = -temp_mesh[i+1][j+1][2]
			temp_mesh[24-i][j+1][3] = temp_mesh[i+1][j+1][3]
			temp_mesh[24-i][j+1][4] = s
			temp_mesh[24-i][j+1][5] = 1.0-temp_mesh[i+1][j+1][5]
			
			v += dv		-- Update angle
		end for
		
		for j=0 to 23 do
			for k=0 to 4 do
				poke(titlescreen_mesh+(i*24*5*4)+(j*5*4)+(k*4),
				     atom_to_float32(temp_mesh[i+1][j+1][k+1]))
				poke(titlescreen_mesh+((23-i)*24*5*4)+(j*5*4)+(k*4),
				     atom_to_float32(temp_mesh[24-i][j+1][k+1]))
			end for
		end for
	end for
	
	temp_mesh = {}
	
	gl_matrixMode(GL_PROJECTION)
	gl_loadIdentity()
	gl_frustum(-1, 1, -1, 1, 1.1, 20)
	glu_lookAt({0, 0, 0},				-- Pos. of camera
	   	   {0, 0, -5},				-- Look at
	   	   {0, 1, 0})				-- Normalvector
	gl_matrixMode(GL_MODELVIEW)
	gl_loadIdentity()


	-- Start playing song #1 
	--music_set_volume(songs[1],#3800)
	--songs[1] = music_play(songs[1],500,3)
	if not PlayMod(songs[1]) then
	end if

	-- Set the callbacks to use
	glut_reshapeFunc(titlescreen_reshape_cb)
	glut_displayFunc(titlescreen_display_cb)
	glut_keyboardFunc(titlescreen_input_cb)
	glut_specialFunc(titlescreen_spec_input_cb)
	joyhandler_id = hjoy_titlescreen_id
end procedure


function titlescreen_display()
	integer offs
	atom t

	t = time()
	speed_factor = (t-last_frame_time)/0.016  -- ~62.5 fps
	last_frame_time = t
	
	gl_clear(GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT)

	-- Move the texture horizontally 
  	gl_matrixMode(GL_TEXTURE)
	gl_pushMatrix()
	gl_translatef({s_trans,s_trans/-4.0,0.0})
	s_trans += 0.00055*speed_factor

	gl_matrixMode(GL_MODELVIEW)
	gl_pushMatrix()
	gl_translatef({0,0,-1.1})

	gl_enable(GL_TEXTURE_2D)
	gl_color3f({color_mult,color_mult,color_mult})

	-- Display the distorted background texture 
	gl_bindTexture(GL_TEXTURE_2D,background)
	gl_begin(GL_QUADS)
		offs = 0
		for i=1 to 23 do
			for j=1 to 23 do
				c_proc(glTexCoord2fv,{titlescreen_mesh+offs+12})
				c_proc(glVertex3fv,{titlescreen_mesh+offs})

				c_proc(glTexCoord2fv,{titlescreen_mesh+offs+492})
				c_proc(glVertex3fv,{titlescreen_mesh+offs+480})

				c_proc(glTexCoord2fv,{titlescreen_mesh+offs+512})
				c_proc(glVertex3fv,{titlescreen_mesh+offs+500})

				c_proc(glTexCoord2fv,{titlescreen_mesh+offs+32})
				c_proc(glVertex3fv,{titlescreen_mesh+offs+20})
				offs += 20
			end for
			offs += 20
		end for
	gl_end()
	
	gl_matrixMode(GL_TEXTURE)
	gl_popMatrix()

	-- Display the phongmap on top, adding a highlight at the center of the screen. 
	gl_enable(GL_BLEND)
	gl_blendFunc(GL_SRC_ALPHA,GL_ONE)
	gl_bindTexture(GL_TEXTURE_2D,phongmap)
	gl_begin(GL_QUADS)
		c_proc(glTexCoord2f,{0,0})
		c_proc(glVertex3f,{-1.1,1.1,-0.05})
		c_proc(glTexCoord2f,{0,1})
		c_proc(glVertex3f,{-1.1,-1.1,-0.05})
		c_proc(glTexCoord2f,{1,1})
		c_proc(glVertex3f,{1.1,-1.1,-0.05})
		c_proc(glTexCoord2f,{1,0})
		c_proc(glVertex3f,{1.1,1.1,-0.05})
	gl_end()

	-- And finally display the Tetris logo, with all the zero-alpha texels masked out. 
	gl_blendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
	gl_bindTexture(GL_TEXTURE_2D,title)
	gl_begin(GL_QUADS)
		c_proc(glTexCoord2f,{0,0.05})
		c_proc(glVertex3f,{-0.7,0.4,-0.04})
		c_proc(glTexCoord2f,{0,0.94})
		c_proc(glVertex3f,{-0.7,-0.35,-0.04})
		c_proc(glTexCoord2f,{1,0.94})
		c_proc(glVertex3f,{0.7,-0.35,-0.04})
		c_proc(glTexCoord2f,{1,0.05})
		c_proc(glVertex3f,{0.7,0.4,-0.04})
	gl_end()

	-- Display the two strings "START GAME" and "OPTIONS"
	for i=0 to 1 do
		if (i+1=choise) then
			c_proc(glColor3f,{0.9,0.2,0.2}*color_mult)
		else
			c_proc(glColor3f,{0.3,0.08,0.08}*color_mult)
		end if
		putsxyz(choises[i+1],
			font1,
			{-0.048*length(choises[i+1]),
			 -0.4-i*0.16,
			 -0.01},
			16,
			{0.12,
			 0.12},
			0.09)
	end for		

	gl_disable(GL_BLEND)
	gl_disable(GL_TEXTURE_2D)

	gl_matrixMode(GL_MODELVIEW)
	gl_popMatrix()
	glut_swapBuffers()

	-- Do the fadeout (if fadespeed has been set)
	if (fade_speed != 0) then
		--music_set_volume(songs[1],floor(color_mult*#3800))  -- Fade the music volume as well

		if (color_mult >= fade_speed) then
			color_mult -= fade_speed
		else
			-- The fadeout is done
			fade_speed = 0.0
			gl_clear(GL_COLOR_BUFFER_BIT)
			-- Delete some textures that aren't needed anymore
			gl_deleteTextures(3,{background,phongmap,title})
			
			--songs[1] = music_stop(songs[1])
			StopMod(songs[1])
			call_proc(start_new_game_id,{})
		end if
	end if
	
	return 0
end function
titlescreen_display_cb = call_back(routine_id("titlescreen_display"))


